#include "config.h"

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "tlswrap.h"

#ifndef HAVE_STRLCPY
#include "strlcpy.h"
#endif

/*
extern char *optarg;
extern int optind;
extern int optopt;
extern int opterr;
extern int optreset;
*/

/*
	Check the "default" egd-pool locations.
*/

void 	egd_check(char *cfg_egdsock, int cfg_egdsock_max)
{
	const char std_loc[3][17]= {"/var/run/egd-pool",
		"/dev/egd-pool", "/etc/egd-pool"};
	int i;

	for (i = 0; i < 3; i++) {
		if (access(std_loc[i], F_OK) == 0) {
			strlcpy(cfg_egdsock, std_loc[i], cfg_egdsock_max);
			return;
		}
	}

	cfg_egdsock[0] = '\0';
}

/*
	Configure everything.
*/

void	read_config(int argc, char * const *argv, unsigned int *users,
	char *listenport, int listenmax, int *debug, char *cfg_egdsock,
	int cfg_egdsock_max, char *tlsciphers, int tlsciphersmax,
	unsigned int *tcpbufsize, unsigned int *tcpsndlowat, char
	*listenhost, int listenhostmax, char *token, int tokenmax)
{
	signed char ch; /* StrongARM fix */
	char *ep;

	/* Set defaults first */

	*users = 5;
	strlcpy(listenport, "7000", listenmax);
	strlcpy(listenhost, "127.0.0.1", listenhostmax);
	strlcpy(token, "#@:", tokenmax);
	*debug = 0;
	*tcpbufsize = 32768;
	*tcpsndlowat = DBUF_SIZE;
	egd_check(cfg_egdsock, cfg_egdsock_max);

	strlcpy(tlsciphers,"RC4-SHA:RC4-MD5:DHE-DSS-RC4-SHA:DES-CBC3-SHA:DES-CBC3-MD5:EDH-RSA-DES-CBC3-SHA:EDH-DSS-DES-CBC3-SHA",
	    tlsciphersmax);

	while ((ch = getopt(argc, argv, "b:B:c:C:dE:h:l:t:")) != -1)
		switch (ch) {
		case 'b':
			*tcpbufsize = strtol(optarg, &ep, 10);
			if (*tcpbufsize <= 0 || *ep != '\0') {
				fprintf(stderr,"illegal number -- %s", optarg);
				exit(1);
			}
		case 'B':
			*tcpsndlowat = strtol(optarg, &ep, 10);
			if (*tcpsndlowat <= 0 || *ep != '\0') {
				fprintf(stderr, "illegal number -- %s", optarg);
				exit(1);
			}
		case 'c':
			*users = strtol(optarg, &ep, 10);
			if (*users <= 0 || *ep != '\0') {
				fprintf(stderr, "illegal number -- %s", optarg);
				exit(1);
			}
			break;
		case 'C':
			strlcpy(tlsciphers, optarg, tlsciphersmax);
			break;
		case 'd':
			*debug = 1;
			break;
		case 'E':
			strlcpy(cfg_egdsock, optarg, cfg_egdsock_max);
			break;
		case 'h':
			strlcpy(listenhost, optarg, listenhostmax);
			break;
		case 'l':
			strlcpy(listenport, optarg, listenmax);
			break;
		case 't':
			if (strlen(optarg) == 3)
				strlcpy(token, optarg, tokenmax);
			else {
				fprintf(stderr, "tokens must be 3 characters\n");
				exit(1);
			}
			break;
		default:
			usage();
		}
		
	argc -= optind;
     	argv += optind;

}
void usage()
{
	(void)fprintf(stderr, "usage: %s [-d] [-c max] [-C list] [-d] [-E socket] [-h host] [-l port] [-t tokens]\n","tlswrap");
	exit(1);
}
